<?php

class Logger {
    const DEBUG = 6;
    const HIGH = 5;
    const NORMAL = 3;
    const LOW = 1;
    const NONE = 0;
    const MONITOR_ID = "xg-portal";

    function __construct($logLevel = Logger::LOW, $logDirectory = "/usr/local/mgportal/log") {
        $this->level = $logLevel;
        $this->directory = $logDirectory;
        $this->logFile = "";
        $this->isOpen = false;
        $this->useHttpLog = false;
        $this->useStdout = false;

        /*
         * Check for invocation mode, output to screen if a "shell".
         */
        if (php_sapi_name() == 'cli') {
            if (isset($_SERVER['TERM'])) {
                $invoked_by = "shell";
                $this->useStdout(true);
            } else {
                $invoked_by = "crontab";
            }
        } else {
            $invoked_by = "other";
        }
        $this->invoked_by = $invoked_by;
    }
    
    function __destruct() {
        closelog();
    }

    public function useHttpLog($useIt) {
        $this->useHttpLog = $useIt;
    }
    
    public function useStdout($useIt) {
        $this->useStdout = $useIt;
    }
    
    private function openLog() {
        if ($this->isOpen == false) {
            openlog(Logger::MONITOR_ID, LOG_NDELAY, LOG_USER);
            $this->isOpen = true;
        }
    }
    
    public function setLogLevel($logLevel) {
        $this->level = $logLevel;
    }

    public function getLogLevel() {
        return $this->level;
    }

    public function setLogFile($file) {
        $this->logFile = $file;
    }

    public function logWarning($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_WARNING, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_WARNING, $this->createLogMessage($message));
        } else {
            file_put_contents($this->logFile, $this->createLogMessage($message, LOG_WARNING, true), FILE_APPEND);   
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logError($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_ERR, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_ERR, $this->createLogMessage($message));
        } else {
            file_put_contents($this->logFile, $this->createLogMessage($message, LOG_ERR, true), FILE_APPEND);
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logNotice($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_NOTICE, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_NOTICE, $this->createLogMessage($message));
        } else {
            file_put_contents($this->logFile, $this->createLogMessage($message, LOG_NOTICE, true), FILE_APPEND);           
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logInfo($message) {
        if ($this->level >= Logger::HIGH) {
            if ($this->useHttpLog) {
                error_log($this->createLogMessage($message, LOG_INFO, false));
            } else if ($this->logFile === "") {
                $this->openLog();
                syslog(LOG_INFO, $this->createLogMessage($message));
            } else {
                file_put_contents($this->logFile, $this->createLogMessage($message, LOG_INFO, true), FILE_APPEND);                
            }
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logDebug($message) {
        if ($this->level >= Logger::DEBUG) {
            if ($this->useHttpLog) {
                error_log($this->createLogMessage($message, LOG_INFO, false));
            } else if ($this->logFile === "") {
                $this->openLog();
                syslog(LOG_INFO, $this->createLogMessage($message));
            } else {
                file_put_contents($this->logFile, $this->createLogMessage($message, LOG_INFO, true), FILE_APPEND);
            }
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function createLogMessage($message, $log_type = "", $addTimestamp = false) {
        // syslogd timestamps all messages.
        if ($addTimestamp) {
            $message = $this->getTimeString () . $log_type . ": " . $message;
        }
        return $message;
    }
    
    private function getTimeString() {
        $timestamp = time();
        $s = date('Y/m/d H:i:s', $timestamp);
        $s .= ": ";
        return $s;
    }

    // Logs a variable with var_dump().
    function writeVariable($var, $argumentNumber = 1, $forceLog = false) {
        if ($forceLog || $this->level >= Logger::HIGH) {
            ob_start();
            var_dump($var);
            $contents = ob_get_contents();
            ob_end_clean();
            $message = "#" . $argumentNumber . ": " . $contents;
            $this->logInfo($message);
        }
    }

    /*
     * Functions for the BP class.
     *  Logs the entrance to BPL.so function, logging name, argument count and values.
     */
    public function enterFunction($stringFunction) {
        if ($this->level >= Logger::HIGH) {
            $message = $this->getTimeString() . " ENTER: " . $stringFunction;
            $nArguments = func_num_args();
            if ($nArguments == 1) {
                $message .= "(no args)\n";
            } else {
                $message .= "(" . ($nArguments - 1) . " args):\n";
            }
            $this->logInfo($message);
            for ($i = 1; $i < $nArguments; $i ++) {
                $arg = func_get_arg($i);
                $this->writeVariable($arg, $i);
            }
        }
    }

    // Logs the exit from BPL.so function, and function result value.
    public function exitFunction($stringFunction, $result) {
        if ($this->level >= Logger::HIGH) {
            $message = $this->getTimeString() . " EXIT: " . $stringFunction;
            $this->logInfo($message);
            $this->writeVariable($result);
        }
    }

}

?>